#include <iostream>
#include <cstring>
#include <string>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <limits>
#include <algorithm>
#include <sys/time.h>
#include <sys/stat.h>
#include <getopt.h> 
#include <cstdlib>
#include <gsl/gsl_sf_gamma.h>

#include "src/THunc_model.hh"
#include "src/StudyDistribution.hh"

using namespace std;



// insert here the information on the perturbative expansion you want to consider

double *initProcess(string &process, int &nOrders, int &Np, double &dmin, double &dmax)
{
  // anharmonic oscillator in quantum mechanics, C. M. Bender and T. T. Wu, Anharmonic oscillator, Phys. Rev. 184 (1969) 1231.
  process = "anharmonic_oscillator";  // name (to be used in output files)
  nOrders = 6;      // number of known orders
  //exactsigma = 1.065285;
  double *Sigma = new double[nOrders];
  double An[9] = {3./4., -21./8., 333./16., -30885./128., 916731./256., -65518401./1024., 2723294673./2048., -1030495099053./32768., 54626982511455./65536.};
  double beta = 0.1;
  for(int n=0; n<nOrders; n++) {
    Sigma[n] = 1;
    for(int k=0; k<n; k++) {
      Sigma[n] += beta*An[k]*pow(beta/2,k);
    }
  }
  //
  Np = 8000;      // the number of points used to sample the distribution in the most important region, defined by the range below
  dmin = 1.04;    // lower limit of the interesting range in units of the LO
  dmax = 1.09;    // upper limit of the interesting range in units of the LO
  //
  return Sigma;
}








double custom_P_deltan_given_pars(double *delta, double *rhat, int k, const double *par) {
  return 0;
}
double custom_P0(const double *par) {
  return 0;
}






void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -P  --PHp <PHp>           set the prior of the hypotesys H+,H- (sign patterns)" << endl
       << endl;
  exit(0);
  return;
}

void read_arguments(int argc, char* argv[], double &probHplus) {
  const char* const short_options = "hP:";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "PHp", 1, NULL, 'P' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 'P':
      probHplus = strtod(optarg, NULL);
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  //
  return;
}


double delta_t(struct timeval t0, struct timeval t1) {
  return t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
}



//
//  Main program
//
int main(int argc, char* argv[])
{
  
  struct timeval t0, t, tlast;
  gettimeofday(&t0,NULL);
  tlast = t0;

  double probHplus = 0;
  read_arguments(argc, argv, probHplus);


  int nOrd;
  double Procdmin=0, Procdmax=2;
  int ProcNp=0;
  string proc;
  ostringstream PHpms;
  if(probHplus>0) PHpms << "_PHpm" << probHplus;

  double *iSigma = initProcess(proc, nOrd, ProcNp, Procdmin, Procdmax);









  int Np = 8800;
  double dmin=-60, dmax=62;
  vector<double> xSigma;
  for(int k=0; k<Np; k++) {
    double d = dmin + k/(Np-1.)*(dmax-dmin);
    xSigma.push_back(d*iSigma[0]);
  }
  //
  for(int k=0; k<ProcNp; k++) {
    double d = Procdmin + k/(ProcNp-1.)*(Procdmax-Procdmin);
    xSigma.push_back(d*iSigma[0]);
  }
  sort(xSigma.begin(), xSigma.end());
  Np = xSigma.size();
  //


  double *valSigma = new double[Np];
  map<string,double*> valPSigmanp1[10];
  map<string,StudyDistribution*> PSigmanp1[10];
  map<string,double> stdev[10];
  map<string,double> xplus68[10], xminus68[10];
  map<string,double> xplus95[10], xminus95[10];
  //map<string,double> Pneg[10];
  for(int n=0; n<nOrd; n++) {
    valPSigmanp1[n]["Geo"] = new double[Np];
  }
  double stdevGeo[10];
  ofstream ofile2, ofile2sum;
  string sorder[] = { "LO", "NLO", "NNLO", "N3LO", "N4LO", "N5LO", "N6LO" };



  gettimeofday(&t,NULL);
  cout << "Step 2 done in " << delta_t(tlast,t) << " s" << endl;
  tlast = t;
  //



  ofile2   .open("output/"+proc+"_P_Sigma_np1"+PHpms.str()+".dat");
  ofile2sum.open("output/"+proc+"_summary"+PHpms.str()+".dat");
  //
  // geometric behaviour model
  THuncModel_Geo    *modelG = new THuncModel_Geo   (iSigma,    nOrd, probHplus);
  //
  //
  for(int n=0; n<nOrd; n++) {
    stdevGeo[n] = iSigma[0]*sqrt(modelG->Var_deltanp1(n));
    for(int k=0; k<Np; k++) {
      valPSigmanp1[n]["Geo"][k] = modelG->P_Sigmanp1(xSigma[k],n);
      if(n==0) valSigma[k] = xSigma[k];
    }
  }


  gettimeofday(&t,NULL);
  cout << "Step 3 done in " << delta_t(tlast,t) << " s" << endl;
  tlast = t;



  for(int n=0; n<nOrd; n++) {
    // study the distributions computed above
    PSigmanp1[n]["Geo"] = new StudyDistribution(valSigma,    valPSigmanp1[n]["Geo"],Np);
    // compute std deviation
    stdev[n]["Geo"] = stdevGeo[n];
    if(stdevGeo[n]<0) stdev[n]["Geo"] = sqrt(PSigmanp1[n]["Geo"]->Moment(2,PSigmanp1[n]["Geo"]->getMean()));
    //
    // compute DoB intervals
    PSigmanp1[n]["Geo"]->DoBinterval(PSigmanp1[n]["Geo"]->getMedian(), 0.68, xplus68[n]["Geo"], xminus68[n]["Geo"]);
    PSigmanp1[n]["Geo"]->DoBinterval(PSigmanp1[n]["Geo"]->getMedian(), 0.95, xplus95[n]["Geo"], xminus95[n]["Geo"]);
    // print results
    cout << "Order n=" << n << endl;
    cout << "           "
         << setw(11) << "Geo model"
	 << endl;
    cout << "  Norm:    "
	 << setw(11) << PSigmanp1[n]["Geo"]->getNorm()
	 << endl;
    cout << "  Mean:    "
	 << setw(11) << PSigmanp1[n]["Geo"]->getMean()
	 << endl;
    cout << "  Mode:    "
	 << setw(11) << PSigmanp1[n]["Geo"]->getMode()
	 << endl;
    cout << "  Median:  "
	 << setw(11) << PSigmanp1[n]["Geo"]->getMedian()
	 << endl;
    cout << "  Stdev:   "
	 << setw(11) << stdev[n]["Geo"]
	 << endl;
    cout << "  SigmaFO: "
	 << setw(11) << iSigma[n]
	 << endl << endl;
    //
    // save results
    ofile2sum << setw(4)  << n
	      << setw(8)  << sorder[n]
	      << setw(18) << "FixedOrder"
	      << setw(14) << iSigma[n]
	      << endl;
    ofile2sum << setw(4)  << n+0.1
	      << setw(8)  << sorder[n]
	      << setw(18) << "GeometricModel"
	      << setw(14) << PSigmanp1[n]["Geo"]->getMean()
	      << setw(14) << PSigmanp1[n]["Geo"]->getMode()
	      << setw(14) << PSigmanp1[n]["Geo"]->getMedian()
	      << setw(14) <<     stdev[n]["Geo"]
	      << setw(14) <<  xminus68[n]["Geo"]
	      << setw(14) <<   xplus68[n]["Geo"]
	      << setw(14) <<  xminus95[n]["Geo"]
	      << setw(14) <<   xplus95[n]["Geo"]
	      << endl;
  }
  ofile2sum.close();
  gettimeofday(&t,NULL);
  cout << "Step 4 done in " << delta_t(tlast,t) << " s" << endl;
  tlast = t;
  //
  //
  for(int k=0; k<Np; k++) {
    ofile2 << setw(12) << xSigma[k];
    for(int n=0; n<nOrd; n++) {
      ofile2 << setw(14) << PSigmanp1[n]["Geo"]->evalDistr(xSigma[k]);
    }
    ofile2 << endl;
  }
  ofile2.close();
  gettimeofday(&t,NULL);
  cout << "Step 5 done in " << delta_t(tlast,t) << " s" << endl;
  tlast = t;



  gettimeofday(&t,NULL);
  double realt = delta_t(t0, t);
  double cput  = clock() / (float)CLOCKS_PER_SEC;
  cout << "Real time elapsed:  " << realt << " s" << endl;
  cout << "CPU  time elapsed:  " << cput  << " s" << endl;
  
  return 0;
  
}
