set style data lines
set clip two
set lmargin at screen 0.13
set bmargin at screen 0.12
set rmargin at screen 0.92
set tmargin at screen 0.91
set xlabel offset -27, 0.8
set ylabel offset 0.8, 0.0
set pointsize 1
set macro
set bars 2


proc='ggH'


set terminal pdfcairo dashed font ", 10" enhanced lw 1 size 3.3,3.
set output 'plots/'.proc.'_uncertainty_summary.pdf'
file = 'output/'.proc.'_summary_muR0.5.dat'
set title 'Higgs production in gluon fusion at LHC 13 TeV, m_H = 125 GeV'
set xrange [-0.6:3.6]
set yrange [0:70]
set xtics ("LO"0, "NLO"1, "NNLO"2, "N^3LO"3, "N^4LO"4, "N^5LO"5)
set xtics offset 0,-0.7
#set xtics nomirror offset 0,-0.6 border rotate by -25
set xlabel 'knowledge of'
set ylabel '{/Symbol S} = cross section  [pb]'
set key bottom right Left reverse
#
#cb='rgbcolor "#666666"'
cb='rgbcolor "#ff2222"'
ca='rgbcolor "#ff8888"'
cc='rgbcolor "#cc6666"'
g ='rgbcolor "#33cc33"'
r ='rgbcolor "#ff4433"'
gr='rgbcolor "#888888"'
#
is0="ceil($1-1e-6)==floor($1+1e-6)          ? $1-0.05 : 0/0"
is1="ceil($1-0.1-1e-6)==floor($1-0.1+1e-6)  ? $1-0.05 : 0/0"
is2="ceil($1-0.2-1e-6)==floor($1-0.2+1e-6)  ? $1-0.15 : 0/0"
is3="ceil($1-0.3-1e-6)==floor($1-0.3+1e-6)  ? $1-0.25 : 0/0"
#
set label 4 'm_H = 125 GeV' at graph 0.75,0.95
set label 3 '{/Symbol m}_F = m_H/2' at graph 0.75,0.9
set label 2 '{/Symbol m} = m_H/2' at graph 0.75,0.85
#
#
set label 1 'Geometric behaviour model' at graph 0.5,0.3
plot file using (@is1):4:7               w yerrorbars     ls 1 lc @cb lw  0.5 pt 7 ps 0     title 'std dev', \
     file using (@is1):10:(0):($11-$10)  w vectors nohead ls 1 lc @cc lw  1.5               title '95% DoB', \
     file using (@is1):8:(0):($9-$8)     w vectors nohead ls 1 lc @ca lw  5.0               title '68% DoB', \
     file using (@is1):6                 w p              ls 1 lc 0   lw  2.5 pt 7 ps 0.3   title 'mean=median', \
     file using (@is0):4:($4-$7):($4+$7) w yerrorbars     ls 1 lc @gr lw  1.5 pt 7 ps 0.4   notitle 'conventional', \
     file using (@is0):4:5:6             w yerrorbars     ls 1 lc 0   lw  1.5 pt 7 ps 0.4   title 'conventional', \
#

set label 1 'Scale variation model' at graph 0.6,0.3
plot file using (@is2):4:7               w yerrorbars     ls 1 lc @cb lw  0.5 pt 7 ps 0     title 'std dev', \
     file using (@is2):10:(0):($11-$10)  w vectors nohead ls 1 lc @cc lw  1.5               title '95% DoB', \
     file using (@is2):8:(0):($9-$8)     w vectors nohead ls 1 lc @ca lw  5.0               title '68% DoB', \
     file using (@is2):6                 w p              ls 1 lc 0   lw  2.5 pt 7 ps 0.3   title 'mean=median', \
     file using (@is0):4:($4-$7):($4+$7) w yerrorbars     ls 1 lc @gr lw  1.5 pt 7 ps 0.4   notitle 'conventional', \
     file using (@is0):4:5:6             w yerrorbars     ls 1 lc 0   lw  1.5 pt 7 ps 0.4   title 'conventional', \
#

set label 1 'Custom model' at graph 0.6,0.3
plot file using (@is3):4:7               w yerrorbars     ls 1 lc @cb lw  0.5 pt 7 ps 0     title 'std dev', \
     file using (@is3):10:(0):($11-$10)  w vectors nohead ls 1 lc @cc lw  1.5               title '95% DoB', \
     file using (@is3):8:(0):($9-$8)     w vectors nohead ls 1 lc @ca lw  5.0               title '68% DoB', \
     file using (@is3):6                 w p              ls 1 lc 0   lw  2.5 pt 7 ps 0.3   title 'mean=median', \
     file using (@is0):4:($4-$7):($4+$7) w yerrorbars     ls 1 lc @gr lw  1.5 pt 7 ps 0.4   notitle 'conventional', \
     file using (@is0):4:5:6             w yerrorbars     ls 1 lc 0   lw  1.5 pt 7 ps 0.4   title 'conventional', \
#




set label 2 '{/Symbol m} independent'
file = 'output/'.proc.'_summary_muRindep.dat'
set label 1 'Geometric behaviour model' at graph 0.5,0.35
plot file using (@is1):4:7               w yerrorbars     ls 1 lc @cb lw  0.5 pt 7 ps 0     title 'std dev', \
     file using (@is1):10:(0):($11-$10)  w vectors nohead ls 1 lc @cc lw  1.5               title '95% DoB', \
     file using (@is1):8:(0):($9-$8)     w vectors nohead ls 1 lc @ca lw  5.0               title '68% DoB', \
     file using (@is1):6                 w p              ls 1 lc @g  lw  2.5 pt 7 ps 0.4   title 'median', \
     file using (@is1):5                 w p              ls 1 lc 1   lw  2.5 pt 7 ps 0.3   title 'mode', \
     file using (@is1):4                 w p              ls 1 lc 0   lw  2.5 pt 7 ps 0.2   title 'mean', \
     file using (@is0):4:($4-$7):($4+$7) w yerrorbars     ls 1 lc @gr lw  1.5 pt 7 ps 0.4   notitle 'conventional', \
     file using (@is0):4:5:6             w yerrorbars     ls 1 lc 0   lw  1.5 pt 7 ps 0.4   title 'conventional', \
#

set label 1 'Scale variation model' at graph 0.6,0.35
plot file using (@is2):4:7               w yerrorbars     ls 1 lc @cb lw  0.5 pt 7 ps 0     title 'std dev', \
     file using (@is2):10:(0):($11-$10)  w vectors nohead ls 1 lc @cc lw  1.5               title '95% DoB', \
     file using (@is2):8:(0):($9-$8)     w vectors nohead ls 1 lc @ca lw  5.0               title '68% DoB', \
     file using (@is2):6                 w p              ls 1 lc @g  lw  2.5 pt 7 ps 0.4   title 'median', \
     file using (@is2):5                 w p              ls 1 lc 1   lw  2.5 pt 7 ps 0.3   title 'mode', \
     file using (@is2):4                 w p              ls 1 lc 0   lw  2.5 pt 7 ps 0.2   title 'mean', \
     file using (@is0):4:($4-$7):($4+$7) w yerrorbars     ls 1 lc @gr lw  1.5 pt 7 ps 0.4   notitle 'conventional', \
     file using (@is0):4:5:6             w yerrorbars     ls 1 lc 0   lw  1.5 pt 7 ps 0.4   title 'conventional', \
#

set label 1 'Custom model' at graph 0.6,0.35
plot file using (@is3):4:7               w yerrorbars     ls 1 lc @cb lw  0.5 pt 7 ps 0     title 'std dev', \
     file using (@is3):10:(0):($11-$10)  w vectors nohead ls 1 lc @cc lw  1.5               title '95% DoB', \
     file using (@is3):8:(0):($9-$8)     w vectors nohead ls 1 lc @ca lw  5.0               title '68% DoB', \
     file using (@is3):6                 w p              ls 1 lc @g  lw  2.5 pt 7 ps 0.4   title 'median', \
     file using (@is3):5                 w p              ls 1 lc 1   lw  2.5 pt 7 ps 0.3   title 'mode', \
     file using (@is3):4                 w p              ls 1 lc 0   lw  2.5 pt 7 ps 0.2   title 'mean', \
     file using (@is0):4:($4-$7):($4+$7) w yerrorbars     ls 1 lc @gr lw  1.5 pt 7 ps 0.4   notitle 'conventional', \
     file using (@is0):4:5:6             w yerrorbars     ls 1 lc 0   lw  1.5 pt 7 ps 0.4   title 'conventional', \
#




quit
