#include "ObservableScaleDep.hh"

using namespace std;

double mZ = 91.1876;
double asZ = 0.118;


/*
	Constructor
 */

QCDobservable::QCDobservable(double * sigma, int nOrders, int offset, double muR, double alphas_muR, int nf) {
  if(offset<0 || offset>2) {
    cout << "Error: the series cannot start at order alphas^" << offset << " because it has not been implmented yet" << endl;
    exit(-3);
  }
  if(nOrders > 6) {
    cout << "Warning: the maximum number of orders implemented is 6. Using 6." << endl;
    nOrders = 6;
  }
  fNOrders = nOrders;
  fSigma   = new double [fNOrders];
  fOffset  = offset;
  fmuR     = muR;
  fnf      = nf;
  AlphaStrong AStest(5, mZ, asZ, 5);
  double asmuR = AStest.alphas(fmuR);
  if(alphas_muR < 0)
    fAlphas = asmuR; // compute the alphas value internally
  else {
    fAlphas = alphas_muR;
    if(alphas_muR < 0.5*asmuR || alphas_muR > 2*asmuR)
      cout << "Are you sure that alpha_s(muR=" << muR << ") = " << alphas_muR << " ?" << endl;
  }
  AS = new AlphaStrong(5, fmuR, fAlphas, fnf);
  //fCoeff[0] = sigma[0]/pow(fAlphas,fOffset);
  //for (int i = 1; i < fNCoeff; ++i)
  //  fCoeff[i] = (sigma[i]-sigma[i-1])/pow(fAlphas,i+fOffset);
  for (int i=0; i<fNOrders; i++)
    fSigma[i] = sigma[i];
}

/*
	Destructor
 */
QCDobservable::~QCDobservable() {
  delete [] fSigma;
}

/*
	Copy Constructor
*/ 
QCDobservable::QCDobservable(const QCDobservable& other):
fNOrders(other.fNOrders),fAlphas(other.fAlphas),fOffset(other.fOffset),fmuR(other.fmuR),fnf(other.fnf) {
  AS = new AlphaStrong(5, fmuR, fAlphas, fnf);
  fSigma = new double [fNOrders];
  for (int i = 0; i < fNOrders; ++i)
    fSigma[i] = other.fSigma[i];
}



void QCDobservable::eval(double mu, double *sigma_out) {
  double b0 = beta0(fnf);
  double b1 = beta1(fnf);
  double b2 = beta2(fnf);
  double b3 = beta3(fnf);
  double b4 = beta4(fnf);
  double asR = AS->alphas(mu); //alphas(mu, fnf, fAlphas, fmuR);
  double as0 = fAlphas;
  double api = asR/M_PI;
  double lr = 2*log(mu/fmuR);
  double factor = asR/as0;
  double xsi[6] = {0,0,0,0,0,0};
  for(int i=0; i<fNOrders; i++) {
    xsi[i] = fSigma[i];
  }
  double LO, NLO, NNLO, N3LO, N4LO, N5LO;
  if(fOffset == 0) {
    LO   = xsi[0];
    NLO  = pow(factor,1) * (xsi[1]-xsi[0]);
    NNLO = pow(factor,2) * (xsi[2]-xsi[1]) + api*b0*NLO *lr;
    N3LO = pow(factor,3) * (xsi[3]-xsi[2]) + api*(api*b1*NLO + 2*b0*NNLO) *lr - api*api*b0*b0*NLO *lr*lr;
    N4LO = pow(factor,4) * (xsi[4]-xsi[3])
      + api*(api*api*b2*NLO + api*2*b1*NNLO + 3*b0*N3LO) *lr
      - api*api*(api*5./2.*b0*b1*NLO + 3*b0*b0*NNLO) *lr*lr
      + api*api*api*b0*b0*b0*NLO *lr*lr*lr;
    N5LO = pow(factor,5) * (xsi[5]-xsi[4])
      + api*(api*api*api*b3*NLO + api*api*2*b2*NNLO + api*3*b1*N3LO + 4*b0*N4LO) *lr
      - api*api*(api*api*(3./2.*b1*b1+3*b0*b2)*NLO + api*7*b0*b1*NNLO + 6*b0*b0*N3LO) *lr*lr
      + api*api*api*(api*13./3.*b0*b0*b1*NLO + 4*b0*b0*b0*NNLO) *lr*lr*lr
      - api*api*api*api*b0*b0*b0*b0*NLO *lr*lr*lr*lr;
  }
  else if(fOffset == 1) {
    LO   = pow(factor,1) * xsi[0];
    NLO  = pow(factor,2) * (xsi[1]-xsi[0]) + api*LO*b0*lr;
    NNLO = pow(factor,3) * (xsi[2]-xsi[1]) + api*(api*b1*LO + 2*b0*NLO) *lr - api*api*b0*b0*LO*lr*lr;
    N3LO = pow(factor,4) * (xsi[3]-xsi[2]) + api*(api*api*b2*LO + api*2*b1*NLO + 3*b0*NNLO) *lr
      - api*api*(api*5./2.*b0*b1*LO + 3*b0*b0*NLO) *lr*lr + api*api*api*b0*b0*b0*LO*lr*lr*lr;
    N4LO = pow(factor,5) * (xsi[4]-xsi[3])
      + api*(api*api*api*b3*LO + api*api*2*b2*NLO + api*3*b1*NNLO + 4*b0*N3LO) *lr
      - api*api*(api*api*(3./2.*b1*b1+3*b0*b2)*LO + api*7*b0*b1*NLO + 6*b0*b0*NNLO) *lr*lr
      + api*api*api*(api*13./3.*b0*b0*b1*LO + 4*b0*b0*b0*NLO) *lr*lr*lr
      - LO*pow(api*b0*lr,4);
    N5LO = pow(factor,6) * (xsi[5]-xsi[4])
      + api*(api*api*api*api*b4*LO + api*api*api*2*b3*NLO + api*api*3*b2*NNLO + api*4*b1*N3LO + 5*b0*N4LO) *lr
      - api*api*(api*api*api*7./2.*(b1*b2+b0*b3)*LO + api*api*(4*b1*b1+8*b0*b2)*NLO + api*27./2.*b0*b1*NNLO + 10*b0*b0*N3LO) *lr*lr
      + api*api*api*(api*api*(35./6.*b0*b1*b1+6*b0*b0*b2)*LO + api*47./3.*b0*b0*b1*NLO + 10*b0*b0*b0*NNLO) *lr*lr*lr
      - api*api*api*api*(api*77./12.*b0*b0*b0*b1*LO + 5*b0*b0*b0*b0*NLO) *lr*lr*lr*lr
      + LO*pow(api*b0*lr,5);
  }
  else if(fOffset == 2) {
    LO   = pow(factor,2) * xsi[0];
    NLO  = pow(factor,3) * (xsi[1]-xsi[0]) + api*LO*2*b0*lr;
    NNLO = pow(factor,4) * (xsi[2]-xsi[1]) + api*(api*2*b1*LO + 3*b0*NLO) *lr - api*api*3*b0*b0*LO*lr*lr;
    N3LO = pow(factor,5) * (xsi[3]-xsi[2]) + api*(api*api*2*b2*LO + api*3*b1*NLO + 4*b0*NNLO) *lr
      - api*api*(api*7*b0*b1*LO + 6*b0*b0*NLO) *lr*lr + api*api*api*4*b0*b0*b0*LO*lr*lr*lr;
    N4LO = pow(factor,6) * (xsi[4]-xsi[3])
      + api*(api*api*api*2*b3*LO + api*api*3*b2*NLO + api*4*b1*NNLO + 5*b0*N3LO) *lr
      - api*api*(api*api*(4*b1*b1+8*b0*b2)*LO + api*27./2.*b0*b1*NLO + 10*b0*b0*NNLO) *lr*lr
      + api*api*api*(api*47./3.*b0*b0*b1*LO + 10*b0*b0*b0*NLO) *lr*lr*lr
      - 5*LO*pow(api*b0*lr,4);
    N5LO = pow(factor,7) * (xsi[5]-xsi[4])
      + api*(api*api*api*api*2*b4*LO + api*api*api*3*b3*NLO + api*api*4*b2*NNLO + api*5*b1*N3LO + 6*b0*N4LO) *lr
      - api*api*(api*api*api*9*(b1*b2+b0*b3)*LO + api*api*(15./2.*b1*b1+15*b0*b2)*NLO + api*22*b0*b1*NNLO + 15*b0*b0*N3LO) *lr*lr
      + api*api*api*(api*api*(59./3.*b0*b1*b1+20*b0*b0*b2)*LO + api*37*b0*b0*b1*NLO + 20*b0*b0*b0*NNLO) *lr*lr*lr
      - api*api*api*api*(api*57./2.*b0*b0*b0*b1*LO + 15*b0*b0*b0*b0*NLO) *lr*lr*lr*lr
      + 6*LO*pow(api*b0*lr,5);
  }
  else {
    cout << "Error: offset not available" << endl;
    exit(-4);
  }
  double XS[6] = {LO, NLO, NNLO, N3LO, N4LO, N5LO};
  //sigma_out = new double[fNOrders];
  for(int i=0; i<fNOrders; i++) {
    sigma_out[i] = 0;
    for(int j=0; j<=i; j++) {
      sigma_out[i] += XS[j];
    }
  }
  double res = sigma_out[fNOrders-1];
  if(res != res) cout << "Got a nan for muR = " << mu << endl;
  return;
}

void QCDobservable::eval(double mu, double *sigma_out, double *r, double factor, unsigned int nScaleVars) {
  double osigma[2*nScaleVars+1][fNOrders];
  int nSV = static_cast<int>(nScaleVars);
  for(int i=-nSV; i<=nSV; i++) {
  //for(int i=0; i<=nSV; i++) {
    double muR = pow(factor, float(i)/nSV) * mu;
    this->eval(muR,osigma[i+nSV]);
  }
  //double sigmadiff[fNOrders][2*nSV+1]; // reverse order, for simpler computations
  for(int k=0; k<fNOrders; k++) {
    sigma_out[k] = osigma[nSV][k];
    vector<double> sigmadiff;
    for(int i=-nSV; i<=nSV; i++) {
    //for(int i=1; i<=nSV; i++) {
      if(i==0) continue;
      //sigmadiff[k][i+nSV] = fabs(osigma[i+nSV][k] - sigma_out[k]);
      sigmadiff.push_back(fabs((osigma[i+nSV][k] - sigma_out[k])/i)*nSV);
    }
    //r[k] = Max(sigmadiff[k]);
    //r[k] = Max(sigmadiff);
    r[k] = Max(sigmadiff)/log(factor);
  }
  return;
}



void QCDobservable::ConventionalScaleUncertainty(double mu, double *err_up, double *err_down, double factor, unsigned int nScaleVars) {
  double osigma[2*nScaleVars+1][fNOrders];
  int nSV = static_cast<int>(nScaleVars);
  for(int i=-nSV; i<=nSV; i++) {
    double muR = pow(factor, float(i)/nSV) * mu;
    this->eval(muR,osigma[i+nSV]);
  }
  for(int k=0; k<fNOrders; k++) {
    vector<double> sigmadiff;
    for(int i=-nSV; i<=nSV; i++) {
      if(i==0) continue;
      sigmadiff.push_back(osigma[nSV+i][k] - osigma[nSV][k]);
    }
    err_up  [k] = Max(sigmadiff);
    err_down[k] = Min(sigmadiff);
  }
  return;
}

