#pragma once

#include <cmath>
#include <cstdlib>
#include <iostream>
#include <fstream>

#include "common.hh"
#include "AlphaStrong.hh"

using namespace std;


extern double alphas(double mu, int nf, double as0, double mu0);


class QCDobservable {

public:
  // sigma is an array of size nOrders of partial sums computed at the renormalization scale_muR, starting at order alphas^offset, with nf active flavours
  QCDobservable(double *sigma, int nOrders, int offset, double scale_muR, double alphas_muR=-1, int nf=5);
  // here instead scale dependence is given as an input, and not computed by the class
  //QCDobservable(double **sigma, int nOrders, unsigned int offset, double *scale_muR, int Npoints);
  QCDobservable(const QCDobservable& other);
  ~QCDobservable();

  int GetNumOrders() { return fNOrders; }
  int GetNf() { return fnf; }
  int GetOffset() { return fOffset; }
  double alphas(double mu) { return AS->alphas(mu); }
  void eval(double scale_muR, double *sigma); // returns an array with sigma at every order
  void eval(double scale_muR, double *sigma, double *r, double factor=4., unsigned int nScaleVars=8); // also returns scale variations

  void ConventionalScaleUncertainty(double scale_muR, double *err_up, double *err_down, double factor=2., unsigned int nScaleVars=1);

private:

  int fNOrders;
  double *fSigma;
  double fAlphas;
  int fOffset;
  double fmuR;
  int fnf;
  AlphaStrong *AS;
  QCDobservable& operator=( const QCDobservable& other );


};
