#pragma once

#include <cmath>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <map>
#include "common.hh"
#include "ObservableScaleDep.hh"
#include "THunc_model.hh"

using namespace std;

enum MuPriorType {
  flat, gaussian, triangle
};


template <class T>
class ScaleIndependentResult {
public:
  ScaleIndependentResult(QCDobservable *obs);
  ScaleIndependentResult(double *sigma, int nOrder, unsigned int offset, double scale_muR, double alphas_muR=-1, int nf=5);
  //ScaleIndependentResult(const ScaleIndependentResult& other);
  ~ScaleIndependentResult();
  //
  // at the moment this must be called mandatorily
  void SetMuPrior(MuPriorType type, double muR_min, double muR_max, double muR_cen=-1); // if muR_cen<0, then the center of the range is used. This argument is irrelevant for flat prior
  //
  void SetPHplusminus(double prob=0.333); // set the probability of the H+ and H- hypothesis
  //
  // expectation values
  double E_Sigman(int n);
  double E_SigmanSq(int n);
  double E_Sigmanp1(int n);
  double E_Sigmanp1Sq(int n);
  double E_Sigmanp1Sq_symm(int n); // more robust, but valid only if the distribution is symmetric in delta_k
  //
  // probability distributions
  double P_Sigmanp1(double Sigma_np1, int n);
  double P_Sigmanp2(double Sigma_np2, int n);
  double P_mu_given_deltan_dots_delta1(double mu, int n);

private:
  int fNOrders, fNmuPoints;
  QCDobservable *fobs;
  T **THmodel;
  double *fmu, **fsigma;
  //double *fdelta, *fdSigma, *frk, *frhatk;
  double fPHplus;
  //
  MuPriorType fPriorType;
  double fpriormumin, fpriormumax, fpriormucen, fpriorwidth;
  //
  double* fDen;
  //
  ScaleIndependentResult& operator=( const ScaleIndependentResult& other );
  //
  void checkN(int n);
  double fPrior(double mu);
  double computeIntegral();
  template<typename U>
  friend int Esigmanp1_integrand  (const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
  template<typename U>
  friend int Esigmanp1Sq_integrand(const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
  template<typename U>
  friend int Psigmanp2_integrand  (const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
  //
  double fNlocal, fSig;
  string fmodelint, fIid;
};




#include "ScaleIndependentResult.inl"
