//#include "ScaleIndependentResult.hh"
#include <vector>
#include <cuba.h>

using namespace std;






//
//  Constructor
//
template <class T>
ScaleIndependentResult<T>::ScaleIndependentResult(double *sigma, int nOrders, unsigned int offset, double scale_muR, double alphas_muR, int nf) {
  fNOrders = nOrders;
  fobs = new QCDobservable(sigma, nOrders, offset, scale_muR, alphas_muR, nf);
  fDen = new double[fNOrders];
  fPHplus = 0;
}
template <class T>
ScaleIndependentResult<T>::ScaleIndependentResult(QCDobservable *obs) {
  fobs = new QCDobservable(*obs);
  fNOrders = obs->GetNumOrders();
  fDen = new double[fNOrders];
  fPHplus = 0;
}

//
//  Destructor
//
template <class T>
ScaleIndependentResult<T>::~ScaleIndependentResult() {
  delete fobs;
}

//
//  Copy Constructor
//
//ScaleIndependentResult::ScaleIndependentResult(const ScaleIndependentResult& other):fNOrders(other.fNOrders) {
//  fNOrders = other.fNOrders;
//  fobs = new QCDobservable(*other.fobs);
//  fpriortype = other.fpriortype;
//  fpriormumin = other.fpriormumin;
//  fpriormumax = other.fpriormumax;
//  fNmuPoints = other.fNmuPoints;
//}

template <class T>
void ScaleIndependentResult<T>::checkN(int n) {
  if(n+1>fNOrders) {
    cout << "Error: you are using a too large value of n (n=" << n << " > nOrders-1=" << fNOrders-1 << ")" << endl;
    exit(5);
  }
  return;
}
template <class T>
void ScaleIndependentResult<T>::SetPHplusminus(double prob) {
  if(prob>1||prob<0) {
    cout << "Setting the probability of the H+ hypotesis outside the [0,1] range." << endl;
    exit(2);
  }
  fPHplus = prob;
  return;
}


inline double Simpson(int i, int n) {
  return n/(n+1.);  // hack to switch off Simpson
  if(i==0 || i==n) return 1./3.;
  return (i%2==0 ? 2. : 4.) /3.;
}
template <class T>
void ScaleIndependentResult<T>::SetMuPrior(MuPriorType type, double muR_min, double muR_max, double muR_cen) {
  fPriorType = type;
  fpriormumin = muR_min;
  fpriormumax = muR_max;
  fpriormucen = muR_cen;
  if(muR_cen<0) fpriormucen = sqrt(muR_min*muR_max); // geometric average
  fpriorwidth = log(fpriormumax/fpriormumin);
  fNmuPoints = int(muR_max/muR_min)*5;
  if(fNmuPoints%2!=0) fNmuPoints++;  // in this way the number of points is even, which is needed for Simpson integration
  THmodel = new T*     [fNmuPoints+1];
  fmu     = new double [fNmuPoints+1];
  fsigma  = new double*[fNmuPoints+1];
  for(int n=0; n<fNOrders; n++) {
    fDen[n] = 0;
  }
  for(int i=0; i<=fNmuPoints; i++) {
    fmu[i] = fpriormumin * exp((i+0.5)*log(fpriormumax/fpriormumin)/(fNmuPoints+1.) ); // better for multi-rectangle integration
    //fmu[i] = fpriormumin * exp(i*log(fpriormumax/fpriormumin)/(fNmuPoints-0.) );  // needed for Simpson integration
    fsigma[i] = new double[fNOrders];
    //double *r = new double[fNOrders];
    fobs->eval(fmu[i], fsigma[i]);
    THmodel[i] = new T(fobs, fmu[i], fPHplus);
    //THmodel[i] = new T(fsigma[i], fNOrders, fPHplus);
    for(int n=0; n<fNOrders; n++) {
      fDen[n] += Simpson(i,fNmuPoints) * fmu[i]*fPrior(fmu[i]) * THmodel[i]->P_deltan_dots_delta1(n);
    }
  }
}
template <class T>
double ScaleIndependentResult<T>::fPrior(double mu) {
  if(mu>fpriormumax) return 0;
  if(mu<fpriormumin) return 0;
  double res = 0;
  // the normalization is not really needed, unless one wants to plot a (normalized) posterior for mu
  //if(fPriorType == flat) res = 1./(fpriormumax-fpriormumin);  // flat in mu
  if(fPriorType == flat)     res = 1./mu/fpriorwidth;  // flat in log mu
  if(fPriorType == triangle) res = 2./mu/fpriorwidth*(mu<fpriormucen ? log(mu/fpriormumin)/log(fpriormucen/fpriormumin) : log(fpriormumax/mu)/log(fpriormumax/fpriormucen));  // in log mu
  if(fPriorType == gaussian) res = exp(-SQR(log(mu/fpriormucen)/fpriorwidth*4)/2)/mu;  // in log mu, with sigma=width/4 (namely, the gaussian is truncated at 2 sigma on each side)
  return res;
}


template <class T>
double ScaleIndependentResult<T>::P_Sigmanp1(double Sigma_np1, int n) {
  checkN(n);
  double num=0, den=fDen[n];
  for(int i=0; i<=fNmuPoints; i++) {
    double delta_np1 = (Sigma_np1-fsigma[i][n])/fsigma[i][0];
    num += Simpson(i,fNmuPoints) * fmu[i]*fPrior(fmu[i]) * THmodel[i]->P_deltanp1_dots_delta1(delta_np1,n) / fsigma[i][0];
  }
  double res = num/den;
  return res;
}

template <class T>
double ScaleIndependentResult<T>::E_Sigman(int n) {
  checkN(n);
  double num=0, den=fDen[n];
  for(int i=0; i<=fNmuPoints; i++) {
    num += Simpson(i,fNmuPoints) * fmu[i]*fPrior(fmu[i]) * THmodel[i]->P_deltan_dots_delta1(n) * fsigma[i][n];
  }
  double res = num/den;
  return res;
}
template <class T>
double ScaleIndependentResult<T>::E_SigmanSq(int n) {
  checkN(n);
  double num=0, den=fDen[n];
  for(int i=0; i<=fNmuPoints; i++) {
    num += Simpson(i,fNmuPoints) * fmu[i]*fPrior(fmu[i]) * THmodel[i]->P_deltan_dots_delta1(n) * SQR(fsigma[i][n]);
  }
  double res = num/den;
  return res;
}
template <class T>
double ScaleIndependentResult<T>::E_Sigmanp1Sq_symm(int n) { // valid only if the distribution is symmetric in delta_k
  checkN(n);
  double num=0, den=fDen[n];
  for(int i=0; i<=fNmuPoints; i++) {
    num += Simpson(i,fNmuPoints) * fmu[i]*fPrior(fmu[i]) * THmodel[i]->P_deltan_dots_delta1(n) * ( SQR(fsigma[i][n]) + SQR(fsigma[i][0])*THmodel[i]->Var_deltanp1(n) );
  }
  double res = num/den;
  return res;
}

template <class T>
int Esigmanp1_integrand(const int *ndim, const double x[], const int *ncomp, double res[], void *pars) {
  ScaleIndependentResult<T> *sir = (ScaleIndependentResult<T> *) pars;
  int n = sir->fNlocal;
  //
  double logmumax = log(sir->fpriormumax/sir->fpriormumin);
  double mu = sir->fpriormumin * exp(logmumax*x[0]);
  double deltanp1 = 10*(x[1]-0.5);
  //
  double *sigma = new double[sir->fNOrders];
  double *r     = new double[sir->fNOrders];
  sir->fobs->eval(mu, sigma, r);
  T *model;
  model = new T(sir->fobs, mu, sir->fPHplus);
  // this check must be updated - it now may depend on epsilon
  if(n==0 && deltanp1==0) deltanp1 = 1e-40;  // in zero the distribution is infinite, but it's integrable, so this trick should fix it
  res[0] = mu*sir->fPrior(mu) * model->P_deltanp1_dots_delta1(deltanp1,n) * (sigma[n] + sigma[0]*deltanp1);
  res[0] *= logmumax * 10;
  //if(res[0]!=res[0]) cout << mu << "   " << deltanp1 << endl;
  //if(1/res[0]==0)    cout << mu << "   " << deltanp1 << endl;
  //
  return 0;
}
template <class T>
int Esigmanp1Sq_integrand(const int *ndim, const double x[], const int *ncomp, double res[], void *pars) {
  ScaleIndependentResult<T> *sir = (ScaleIndependentResult<T> *) pars;
  int n = sir->fNlocal;
  //
  double logmumax = log(sir->fpriormumax/sir->fpriormumin);
  double mu = sir->fpriormumin * exp(logmumax*x[0]);
  double deltanp1 = 10*(x[1]-0.5);
  //
  double *sigma = new double[sir->fNOrders];
  double *r     = new double[sir->fNOrders];
  sir->fobs->eval(mu, sigma, r);
  T *model;
  model = new T(sir->fobs, mu, sir->fPHplus);
  // this check must be updated - it now may depend on epsilon
  if(n==0 && deltanp1==0) deltanp1 = 1e-40;  // in zero the distribution is infinite, but it's integrable, so this trick should fix it
  res[0] = mu*sir->fPrior(mu) * model->P_deltanp1_dots_delta1(deltanp1,n) * pow(sigma[n] + sigma[0]*deltanp1, 2);
  res[0] *= logmumax * 10;
  //if(res[0]!=res[0]) cout << mu << "   " << deltanp1 << endl;
  //if(1/res[0]==0)    cout << mu << "   " << deltanp1 << endl;
  //
  return 0;
}
template <class T>
int Psigmanp2_integrand(const int *ndim, const double x[], const int *ncomp, double res[], void *pars) {
  ScaleIndependentResult<T> *sir = (ScaleIndependentResult<T> *) pars;
  int n = sir->fNlocal;
  double logmumax = log(sir->fpriormumax/sir->fpriormumin);
  double mu = sir->fpriormumin * exp(logmumax*x[0]);
  double *sigma = new double[sir->fNOrders];
  double *r     = new double[sir->fNOrders];
  sir->fobs->eval(mu, sigma, r);
  double deltanp1 = 10*(x[1]-0.5);
  double deltanp2 = (sir->fSig - sigma[n])/sigma[0] - deltanp1;
  //
  T *model;
  model = new T(sir->fobs, mu, sir->fPHplus);
  // this check must be updated - it now may depend on epsilon
  if(n==0 && deltanp1==0) deltanp1 = 1e-40;  // in zero the distribution is infinite, but it's integrable, so this trick should fix it
  //
  res[0] = mu*sir->fPrior(mu) * model->P_deltanp2_dots_delta1(deltanp2,deltanp1,n) / sigma[0];
  res[0] *= logmumax * 10;
  //if(res[0]!=res[0]) cout << mu << "   " << deltanp1 << endl;
  //if(1/res[0]==0)    cout << mu << "   " << deltanp1 << endl;
  //
  return 0;
}
template <class T>
double ScaleIndependentResult<T>::computeIntegral() {
  double cres[1], error[1], prob[1];
  double epsrel=1e-2, epsabs=1.e-6;
  int mineval = 0, maxeval = 50000;
  int last = 4;
  int verbose = 0;
  int nregions, neval, fail, seed;
  nregions = 0;
  seed = 0;
  //system("export CUBACORES=8");
  int dim = 2, ncomp = 1;
  if(fIid == "Esigmanp1") {
    Cuhre(dim, ncomp, Esigmanp1_integrand<T>, this, 1, epsrel, epsabs, verbose | last,       mineval, maxeval, 13, NULL, NULL, &nregions, &neval, &fail, cres, error, prob);
    //Vegas(dim, ncomp, Esigmanp1_integrand<T>, this, 1, epsrel, epsabs, verbose | last, seed, mineval, maxeval, 50000, 20000, 20000, 1, NULL, NULL, &neval, &fail, cres, error, prob);
    //Suave(dim, ncomp, Esigmanp1_integrand<T>, this, 1, epsrel, epsabs, verbose | last, seed, mineval, maxeval, 50000, 20000, 1, NULL, NULL, &nregions, &neval, &fail, cres, error, prob);
  }
  else if(fIid == "Esigmanp1Sq") {
    Cuhre(dim, ncomp, Esigmanp1Sq_integrand<T>, this, 1, epsrel, epsabs, verbose | last,       mineval, maxeval, 13, NULL, NULL, &nregions, &neval, &fail, cres, error, prob);
    //Vegas(dim, ncomp, Esigmanp1_integrand<T>, this, 1, epsrel, epsabs, verbose | last, seed, mineval, maxeval, 50000, 20000, 20000, 1, NULL, NULL, &neval, &fail, cres, error, prob);
    //Suave(dim, ncomp, Esigmanp1_integrand<T>, this, 1, epsrel, epsabs, verbose | last, seed, mineval, maxeval, 50000, 20000, 1, NULL, NULL, &nregions, &neval, &fail, cres, error, prob);
  }
  else if(fIid == "Psigmanp2") {
    Cuhre(dim, ncomp, Psigmanp2_integrand<T>, this, 1, epsrel, epsabs, verbose | last,       mineval, maxeval, 13, NULL, NULL, &nregions, &neval, &fail, cres, error, prob);
  }
  else {
    cout << "Error - name of the integrand has been mistyped" << endl;
    fail = 0;
  }
  //
  if(fail == -1) {
    cout << "\033[0;31m  ERROR: dimension out of range.\033[0m" << endl;
  } else if(fail>0) {
    cout << "\033[0;31m  ERROR: precision not reached.\033[0m" << endl;
  }
  return cres[0];
}
template <class T>
double ScaleIndependentResult<T>::E_Sigmanp1(int n) {
  checkN(n);
  fNlocal = n;
  fIid = "Esigmanp1";
  double num = computeIntegral();
  double den = fDen[n] * fpriorwidth / fNmuPoints;
  double res = num/den;
  return res;
}
template <class T>
double ScaleIndependentResult<T>::E_Sigmanp1Sq(int n) {
  checkN(n);
  fNlocal = n;
  fIid = "Esigmanp1Sq";
  double num = computeIntegral();
  double den = fDen[n] * fpriorwidth / fNmuPoints;
  double res = num/den;
  return res;
}
template <class T>
double ScaleIndependentResult<T>::P_Sigmanp2(double Sigma_np2, int n) {
  checkN(n);
  //*
  double num=0, den=fDen[n];
  for(int i=0; i<=fNmuPoints; i++) {
    num += Simpson(i,fNmuPoints) * fmu[i]*fPrior(fmu[i]) * THmodel[i]->P_Sigmanp2(Sigma_np2,n) * THmodel[i]->P_deltan_dots_delta1(n);
  }
  double res = num/den;
  return res;
  /*/
  // 2d integral version
  fSig = Sigma_np2;
  fNlocal = n;
  fIid = "Psigmanp2";
  double num = computeIntegral();
  double den = fDen[n] * fpriorwidth / fNmuPoints;
  double res = num/den;
  return res;
  //*/
}


template <class T>
double ScaleIndependentResult<T>::P_mu_given_deltan_dots_delta1(double mu, int n) {
  checkN(n);
  double num=0, den=fDen[n];
  T* THmodelmu;
  THmodelmu = new T(fobs, mu, fPHplus);
  num = fPrior(mu) * THmodelmu->P_deltan_dots_delta1(n);
  //for(int i=0; i<=fNmuPoints; i++) {
  //  den += fmu[i]*fPrior(fmu[i]) * model[i]->P_deltan_dots_delta1(n);
  //}
  den *= fpriorwidth / fNmuPoints;
  double res = num/den;
  //if(n==0) cout << mu*num << "  " << den << endl;
  return res;
}
