#pragma once

#include <cmath>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <gsl/gsl_spline.h>
#include "common.hh"

using namespace std;


class StudyDistribution {
public:
  StudyDistribution(double *x, double *distr, unsigned int nPoints);
  StudyDistribution(vector<double> x, vector<double> distr);
  StudyDistribution(map<double,double> distr);
  //StudyDistribution(const StudyDistribution& other);
  ~StudyDistribution();

  friend StudyDistribution operator + (const StudyDistribution &a, const StudyDistribution &b);

  void Normalize();

  double evalDistr(double x);
  double evalProb(double x1, double x2);
  double getNorm() { return fNorm; }
  double getMean() { return fMean; }
  double getMode() { return fMode; }
  double getMedian() { return fMedian; }

  double getXmin() { return fx[0]; }
  double getXmax() { return fx[fNpoints-1]; }

  double Moment(int mom, double x0=0);

  void DoBinterval(double x0, double DoB, double &xplus, double &xminus);


private:

  int fNpoints;
  double *fx, *fP;
  gsl_spline* fSpline;
  gsl_interp_accel *acc;
  double fNorm, fMode, fMean, fMedian;

  void InitSpline();
  double computeIntegral(int mom, double x0, double xmin, double xmax);
  double computeIntegral(int mom, double x0) { return computeIntegral(mom, x0, getXmin(), getXmax()); }
  friend double EintegrandGauss(double x, void *par);
  friend double zeroFinding(double x, void *par);
  double computeMedian();

  int fMom, fDirection;
  double fx0, fxcen, fDoB, fTrickNorm;
  //double fNlocal;
  //string fmodelint;
};
